/*
 * BiZduinoWiFi.h - BiZduino ESP_WROOM_02モジュールライブラリ
 * ITEADLIB Arduino WeeESP8266 ライブラリを元に作成
 */

#include "BiZduinoWiFi.h"
#include <arduino.h>


#define OUTPUT_WIFI_RESET   8   // 出力ピン: Wi-Fiのリセット
#define OUTPUT_WIFI_ENABLE  9   // 出力ピン: Wi-Fiの有効化


////////////////////////////////////////////////////////////////////////////////
// パブリック関数

// WiFi設定
// baud     シリアルのボーレート(デフォルト115200)
void BiZduinoWiFi::begin(uint32_t baud)
{
    pinMode(OUTPUT_WIFI_RESET, OUTPUT);
    pinMode(OUTPUT_WIFI_ENABLE, OUTPUT);
    Serial.begin(baud);
    resetWiFi();
    rx_empty();
    enableSerial();
}

// Wi-Fiモジュールをシリアルポートに接続
// return:  true=接続状態に切り替えた / false=すでに接続済
bool BiZduinoWiFi::enableSerial(void)
{
    if (digitalRead(OUTPUT_WIFI_ENABLE) == LOW) {
        rx_empty();
        delay(50);
        digitalWrite(OUTPUT_WIFI_ENABLE, HIGH);
        return true;
    }
    return false;
}

// Wi-Fiモジュールをシリアルポートから切断
// return:  true=切断状態に切り替えた / false=すでに切断済
bool BiZduinoWiFi::disableSerial(void)
{
    if (digitalRead(OUTPUT_WIFI_ENABLE) != LOW) {
        rx_empty();
        digitalWrite(OUTPUT_WIFI_ENABLE, LOW);
        return true;
    }
    return false;
}

// Wi-Fiモジュールをリセットする
void BiZduinoWiFi::resetWiFi(void)
{
    digitalWrite(OUTPUT_WIFI_ENABLE, HIGH);
    delay(100);
    digitalWrite(OUTPUT_WIFI_RESET, LOW);
}


// WiFiモジュール通信確認 ("AT")
// return:  true=成功 / false=エラー
bool BiZduinoWiFi::kick(void)
{
    enableSerial();
    return eAT();
}

// リスタートコマンド ("AT+RST")
// return:  true=成功 / false=エラー
bool BiZduinoWiFi::restart(void)
{
    enableSerial();
    unsigned long start;
    if (eATRST()) {
        delay(2000);
        start = millis();
        while (millis() - start < 3000) {
            if (eAT()) {
                delay(1500); /* Waiting for stable */
                return true;
            }
            delay(100);
        }
    }
    return false;
}

// WiFiモジュールのバージョン情報取得
// return:  バージョン情報
String BiZduinoWiFi::getVersion(void)
{
    enableSerial();
    String version;
    eATGMR(version);
    return version;
}

// ステーションモードに設定する
// return:  true=成功 / false=エラー
bool BiZduinoWiFi::setOprToStation(void)
{
    enableSerial();
    uint8_t mode;
    if (!qATCWMODE(&mode)) {
        return false;
    }
    if (mode == 1) {
        return true;
    } else {
        if (sATCWMODE(1) && restart()) {
            return true;
        } else {
            return false;
        }
    }
}

// ソフトAPモードに設定する
// return:  true=成功 / false=エラー
bool BiZduinoWiFi::setOprToSoftAP(void)
{
    enableSerial();
    uint8_t mode;
    if (!qATCWMODE(&mode)) {
        return false;
    }
    if (mode == 2) {
        return true;
    } else {
        if (sATCWMODE(2) && restart()) {
            return true;
        } else {
            return false;
        }
    }
}

// ステーション+ソフトAPモードに設定する
// return:  true=成功 / false=エラー
bool BiZduinoWiFi::setOprToStationSoftAP(void)
{
    enableSerial();
    uint8_t mode;
    if (!qATCWMODE(&mode)) {
        return false;
    }
    if (mode == 3) {
        return true;
    } else {
        if (sATCWMODE(3) && restart()) {
            return true;
        } else {
            return false;
        }
    }
}

// アクセスポイントリストを取得する
// return:  アクセスポイントリスト文字列
//          ※APが多いとメモリ不足で取得出来ない場合があります
//
// +CWLAP:(<ecn>,"<ssid>",<rssi>,"<mac>",<ch>,<freq offset>,<freq calibration>)
String BiZduinoWiFi::getAPList(void)
{
    enableSerial();
    String list;
    eATCWLAP(list);
    return list;
}

// アクセスポイントに接続する
// ssid     SSID
// pwd      パスワード
// return:  true=成功 / false=エラー
bool BiZduinoWiFi::joinAP(String ssid, String pwd)
{
    enableSerial();
    return sATCWJAP(ssid, pwd);
}

// DHCPモードを設定する
// mode     サーバモード(0=soft AP, 1=station, 2=both)
// enabled  DHCP有効
// return:  true=成功 / false=エラー
bool BiZduinoWiFi::enableClientDHCP(uint8_t mode, boolean enabled)
{
    enableSerial();
    return sATCWDHCP(mode, enabled);
}

// アクセスポイントとの接続を切る
// return:  true=成功 / false=エラー
bool BiZduinoWiFi::leaveAP(void)
{
    enableSerial();
    return eATCWQAP();
}

// ソフトAPモードのパラメータを設定する
// ssid     SSID
// pwd      パスワード
// chi      チャンネル(1～13、デフォルト7)
// ecn      暗号方式 0=暗号なし / 1=WEP / 2=WPA_PSK / 3=WPA2_PSK / 4=WPA_WPA2_PSK(デフォルト)
// return:  true=成功 / false=エラー
bool BiZduinoWiFi::setSoftAPParam(String ssid, String pwd, uint8_t chl, uint8_t ecn)
{
    enableSerial();
    return sATCWSAP(ssid, pwd, chl, ecn);
}

// ソフトAPモードで接続しているデバイスのIPリストを取得する
// return:  アドレスリスト文字列
//
// "<IP addr>","<mac>"
String BiZduinoWiFi::getJoinedDeviceIP(void)
{
    enableSerial();
    String list;
    eATCWLIF(list);
    return list;
}

// 接続ステータスを取得する
// return:  接続ステータス文字列
//
// STATUS:<stat>
// +CIPSTATUS:<link ID>,<type>,<remote_IP>,<remote_port>,<local_port>,<tetype>
String BiZduinoWiFi::getIPStatus(void)
{
    enableSerial();
    String list;
    eATCIPSTATUS(list);
    return list;
}

// ローカルIPリストを取得する
// return:  ローカルIPリスト文字列
//
// +CIFSR:STAIP,"<IP addr>"
// +CIFSR:STAMAC,"<mac>"
String BiZduinoWiFi::getLocalIP(void)
{
    enableSerial();
    String list;
    eATCIFSR(list);
    return list;
}

// マルチコネクションモードにする
// return:  true=成功 / false=エラー
bool BiZduinoWiFi::enableMUX(void)
{
    enableSerial();
    return sATCIPMUX(1);
}

// シングルコネクションモードにする
// return:  true=成功 / false=エラー
bool BiZduinoWiFi::disableMUX(void)
{
    enableSerial();
    return sATCIPMUX(0);
}

// TCP接続(シングルコネクション)
// addr     アドレス
// port     ポート
// return:  true=成功 / false=エラー
bool BiZduinoWiFi::createTCP(String addr, uint32_t port)
{
    enableSerial();
    return sATCIPSTARTSingle("TCP", addr, port);
}

// TCP切断(シングルコネクション)
// return:  true=成功 / false=エラー
bool BiZduinoWiFi::releaseTCP(void)
{
    enableSerial();
    return eATCIPCLOSESingle();
}

// UDP接続(シングルコネクション)
// addr     アドレス
// port     ポート
// return:  true=成功 / false=エラー
bool BiZduinoWiFi::registerUDP(String addr, uint32_t port)
{
    enableSerial();
    return sATCIPSTARTSingle("UDP", addr, port);
}

// UDP切断(シングルコネクション)
// return:  true=成功 / false=エラー
bool BiZduinoWiFi::unregisterUDP(void)
{
    enableSerial();
    return eATCIPCLOSESingle();
}

// TCP接続(マルチコネクション)
// mux_id   マルチコネクションID(0～4)
// addr     アドレス
// port     ポート
// return:  true=成功 / false=エラー
bool BiZduinoWiFi::createTCP(uint8_t mux_id, String addr, uint32_t port)
{
    enableSerial();
    return sATCIPSTARTMultiple(mux_id, "TCP", addr, port);
}

// TCP切断(マルチコネクション)
// mux_id   マルチコネクションID(0～4)
// return:  true=成功 / false=エラー
bool BiZduinoWiFi::releaseTCP(uint8_t mux_id)
{
    enableSerial();
    return sATCIPCLOSEMulitple(mux_id);
}

// UDP接続(マルチコネクション)
// mux_id   マルチコネクションID(0～4)
// addr     アドレス
// port     ポート
// return:  true=成功 / false=エラー
bool BiZduinoWiFi::registerUDP(uint8_t mux_id, String addr, uint32_t port)
{
    enableSerial();
    return sATCIPSTARTMultiple(mux_id, "UDP", addr, port);
}

// UDP切断(マルチコネクション)
// mux_id   マルチコネクションID(0～4)
// return:  true=成功 / false=エラー
bool BiZduinoWiFi::unregisterUDP(uint8_t mux_id)
{
    enableSerial();
    return sATCIPCLOSEMulitple(mux_id);
}

// TCPサーバ時のタイムアウトを設定する
// timeout  タイムアウト秒(0～28800、デフォルト180)
// return:  true=成功 / false=エラー
bool BiZduinoWiFi::setTCPServerTimeout(uint32_t timeout)
{
    enableSerial();
    return sATCIPSTO(timeout);
}

// TCPサーバを開始
// port     ポート(デフォルト333)
// return:  true=成功 / false=エラー
//
// マルチコネクションモードにする必要がある
// クライアント接続情報はgetIPStatus()で取得する。
// サーバの受信待ち受けは
//   uint32_t BiZduinoWiFi::recv(uint8_t *coming_mux_id, uint8_t *buffer, uint32_t buffer_size, uint32_t timeout)
// を使用する
bool BiZduinoWiFi::startTCPServer(uint32_t port)
{
    enableSerial();
    if (sATCIPSERVER(1, port)) {
        return true;
    }
    return false;
}

// TCPサーバを停止
// return:  true=成功 / false=エラー
bool BiZduinoWiFi::stopTCPServer(void)
{
    enableSerial();
    sATCIPSERVER(0);
    restart();
    return false;
}

// TCPサーバを開始
// port     ポート(デフォルト333)
// return:  true=成功 / false=エラー
bool BiZduinoWiFi::startServer(uint32_t port)
{
    return startTCPServer(port);
}

// TCPサーバを停止
// return:  true=成功 / false=エラー
bool BiZduinoWiFi::stopServer(void)
{
    return stopTCPServer();
}

// データ送信(シングルコネクション)
// buffer   データ
// len      データ長
// return:  true=成功 / false=エラー
bool BiZduinoWiFi::send(const uint8_t *buffer, uint32_t len)
{
    enableSerial();
    return sATCIPSENDSingle(buffer, len);
}

// データ送信(マルチコネクション)
// mux_id   マルチコネクションID(0～4)
// buffer   データ
// len      データ長
// return:  true=成功 / false=エラー
bool BiZduinoWiFi::send(uint8_t mux_id, const uint8_t *buffer, uint32_t len)
{
    enableSerial();
    return sATCIPSENDMultiple(mux_id, buffer, len);
}

// データ受信(シングルコネクション)
// buffer       受信バッファ
// buffer_size  受信バッファ長
// timeout      タイムアウト
// return:      受信データ長
uint32_t BiZduinoWiFi::recv(uint8_t *buffer, uint32_t buffer_size, uint32_t timeout)
{
    enableSerial();
    return recvPkg(buffer, buffer_size, NULL, timeout, NULL);
}

// データ受信(マルチコネクション)
// mux_id       マルチコネクションID(0～4)
// buffer       受信バッファ
// buffer_size  受信バッファ長
// timeout      タイムアウト
// return:      受信データ長
uint32_t BiZduinoWiFi::recv(uint8_t mux_id, uint8_t *buffer, uint32_t buffer_size, uint32_t timeout)
{
    enableSerial();
    uint8_t id;
    uint32_t ret;
    ret = recvPkg(buffer, buffer_size, NULL, timeout, &id);
    if (ret > 0 && id == mux_id) {
        return ret;
    }
    return 0;
}

// サーバデータ受信
// coming_mux_id    マルチコネクションID格納先(0～4)
// buffer           受信バッファ
// buffer_size      受信バッファ長
// timeout          タイムアウト
// return:          受信データ長
uint32_t BiZduinoWiFi::recv(uint8_t *coming_mux_id, uint8_t *buffer, uint32_t buffer_size, uint32_t timeout)
{
    enableSerial();
    return recvPkg(buffer, buffer_size, NULL, timeout, coming_mux_id);
}




////////////////////////////////////////////////////////////////////////////////
// プライベート関数



// パッケージデータ受信
// +IPD,<id>,<len>:<data>
// +IPD,<len>:<data>
//
// buffer           受信バッファ
// buffer_size      受信バッファ長(バッファを超えたデータは読み捨て)
// data_len         パッケージデータ長格納先
// timeout          タイムアウト
// coming_mux_id    マルチコネクションID格納先(0～4)
// return:          受信データ長


uint32_t BiZduinoWiFi::recvPkg(uint8_t *buffer, uint32_t buffer_size, uint32_t *data_len, uint32_t timeout, uint8_t *coming_mux_id)
{
    String data;
    char a;
    int32_t index_PIPDcomma = -1;
    int32_t index_colon = -1; /* : */
    int32_t index_comma = -1; /* , */
    int32_t len = -1;
    int8_t id = -1;
    bool has_data = false;
    uint32_t ret;
    unsigned long start;
    uint32_t i;
    
    if (buffer == NULL) {
        return 0;
    }
    
    start = millis();
    while (millis() - start < timeout) {
        if(Serial.available() > 0) {
            a = Serial.read();
            data += a;
        }
        
        index_PIPDcomma = data.indexOf("+IPD,");
        if (index_PIPDcomma != -1) {
            index_colon = data.indexOf(':', index_PIPDcomma + 5);
            if (index_colon != -1) {
                index_comma = data.indexOf(',', index_PIPDcomma + 5);
                /* +IPD,id,len:data */
                if (index_comma != -1 && index_comma < index_colon) { 
                    id = data.substring(index_PIPDcomma + 5, index_comma).toInt();
                    if (id < 0 || id > 4) {
                        return 0;
                    }
                    len = data.substring(index_comma + 1, index_colon).toInt();
                    if (len <= 0) {
                        return 0;
                    }
                } else { /* +IPD,len:data */
                    len = data.substring(index_PIPDcomma + 5, index_colon).toInt();
                    if (len <= 0) {
                        return 0;
                    }
                }
                has_data = true;
                break;
            }
        }
    }
    
    if (has_data) {
        i = 0;
        ret = len > buffer_size ? buffer_size : len;
        start = millis();
        while (millis() - start < 3000) {
            while(Serial.available() > 0 && i < ret) {
                a = Serial.read();
                buffer[i++] = a;
            }
            if (i == ret) {
                rx_empty();
                if (data_len) {
                    *data_len = len;    
                }
                if (index_comma != -1 && coming_mux_id) {
                    *coming_mux_id = id;
                }
                return ret;
            }
        }
    }
    return 0;
}

// 受信データが無くなるまで読み捨てる
void BiZduinoWiFi::rx_empty(void) 
{
    while(Serial.available() > 0) {
        Serial.read();
    }
}

// 指定文字列まで受信
// target   受信終了文字列
// timeout  タイムアウト
// return:  受信文字列
String BiZduinoWiFi::recvString(String target, uint32_t timeout)
{
    String data;
    char a;
    unsigned long start = millis();
    while (millis() - start < timeout) {
        while(Serial.available() > 0) {
            a = Serial.read();
            if(a == '\0') continue;
            data += a;
        }
        if (data.indexOf(target) != -1) {
            break;
        }   
    }
    return data;
}

// 指定文字列まで受信
// target1  受信終了文字列
// target2  受信終了文字列
// timeout  タイムアウト
// return:  受信文字列
String BiZduinoWiFi::recvString(String target1, String target2, uint32_t timeout)
{
    String data;
    char a;
    unsigned long start = millis();
    while (millis() - start < timeout) {
        while(Serial.available() > 0) {
            a = Serial.read();
            if(a == '\0') continue;
            data += a;
        }
        if (data.indexOf(target1) != -1) {
            break;
        } else if (data.indexOf(target2) != -1) {
            break;
        }
    }
    return data;
}

// 指定文字列まで受信
// target1  受信終了文字列
// target2  受信終了文字列
// target3  受信終了文字列
// timeout  タイムアウト
// return:  受信文字列
String BiZduinoWiFi::recvString(String target1, String target2, String target3, uint32_t timeout)
{
    String data;
    char a;
    unsigned long start = millis();
    while (millis() - start < timeout) {
        while(Serial.available() > 0) {
            a = Serial.read();
            if(a == '\0') continue;
            data += a;
        }
        if (data.indexOf(target1) != -1) {
            break;
        } else if (data.indexOf(target2) != -1) {
            break;
        } else if (data.indexOf(target3) != -1) {
            break;
        }
    }
    return data;
}

// 指定文字列の受信を待つ
// target   受信終了文字列
// timeout  タイムアウト
// return:  true=指定文字列を受信した / false=タイムアウト
bool BiZduinoWiFi::recvFind(String target, uint32_t timeout)
{
    String data_tmp;
    data_tmp = recvString(target, timeout);
    if (data_tmp.indexOf(target) != -1) {
        return true;
    }
    return false;
}

// 指定終了文字列まで受信して、指定文字列間の文字列を切り出す
// target   受信終了文字列
// begin    切り出し開始文字列
// end      切り出し終了文字列
// data     切り出し文字列格納先
// timeout  タイムアウト
// return:  true=切り出し成功 / false=タイムアウトもしくは切り出し該当なし
bool BiZduinoWiFi::recvFindAndFilter(String target, String begin, String end, String &data, uint32_t timeout)
{
    String data_tmp;
    data_tmp = recvString(target, timeout);
    if (data_tmp.indexOf(target) != -1) {
        int32_t index1 = data_tmp.indexOf(begin);
        int32_t index2 = data_tmp.indexOf(end);
        if (index1 != -1 && index2 != -1) {
            index1 += begin.length();
            data = data_tmp.substring(index1, index2);
            return true;
        }
    }
    data = "";
    return false;
}


////////// 以下各ATコマンド処理

bool BiZduinoWiFi::eAT(void)
{
    rx_empty();
    Serial.println(F("AT"));
    return recvFind(F("OK"));
}

bool BiZduinoWiFi::eATRST(void) 
{
    rx_empty();
    Serial.println(F("AT+RST"));
    return recvFind(F("OK"));
}

bool BiZduinoWiFi::eATGMR(String &version)
{
    rx_empty();
    Serial.println(F("AT+GMR"));
    return recvFindAndFilter(F("OK"), F("\r\r\n"), F("\r\nOK"), version); 
}

bool BiZduinoWiFi::qATCWMODE(uint8_t *mode) 
{
    String str_mode;
    bool ret;
    if (!mode) {
        return false;
    }
    rx_empty();
    Serial.println(F("AT+CWMODE?"));
    ret = recvFindAndFilter(F("OK"), F("+CWMODE:"), F("\r\n\r\nOK"), str_mode); 
    if (ret) {
        *mode = (uint8_t)str_mode.toInt();
        return true;
    } else {
        return false;
    }
}

bool BiZduinoWiFi::sATCWMODE(uint8_t mode)
{
    String data;
    rx_empty();
    Serial.print(F("AT+CWMODE="));
    Serial.println(mode);
    
    data = recvString(F("OK"), F("no change"));
    if (data.indexOf(F("OK")) != -1 || data.indexOf(F("no change")) != -1) {
        return true;
    }
    return false;
}

bool BiZduinoWiFi::sATCWJAP(String ssid, String pwd)
{
    String data;
    rx_empty();
    Serial.print(F("AT+CWJAP=\""));
    Serial.print(ssid);
    Serial.print(F("\",\""));
    Serial.print(pwd);
    Serial.println(F("\""));
    
    data = recvString(F("OK"), F("FAIL"), 10000);
    if (data.indexOf(F("OK")) != -1) {
        return true;
    }
    return false;
}

bool BiZduinoWiFi::sATCWDHCP(uint8_t mode, boolean enabled)
{
    String strEn = "0";
    if (enabled) {
        strEn = "1";
    }
    
    
    String data;
    rx_empty();
    Serial.print(F("AT+CWDHCP="));
    Serial.print(strEn);
    Serial.print(F(","));
    Serial.println(mode);
    
    data = recvString(F("OK"), F("FAIL"), 10000);
    if (data.indexOf(F("OK")) != -1) {
        return true;
    }
    return false;
}

bool BiZduinoWiFi::eATCWLAP(String &list)
{
    String data;
    rx_empty();
    Serial.println(F("AT+CWLAP"));
    return recvFindAndFilter(F("OK"), F("\r\r\n"), F("\r\n\r\nOK"), list, 10000);
}

bool BiZduinoWiFi::eATCWQAP(void)
{
    String data;
    rx_empty();
    Serial.println(F("AT+CWQAP"));
    return recvFind(F("OK"));
}

bool BiZduinoWiFi::sATCWSAP(String ssid, String pwd, uint8_t chl, uint8_t ecn)
{
    String data;
    rx_empty();
    Serial.print(F("AT+CWSAP=\""));
    Serial.print(ssid);
    Serial.print(F("\",\""));
    Serial.print(pwd);
    Serial.print(F("\","));
    Serial.print(chl);
    Serial.print(F(","));
    Serial.println(ecn);
    
    data = recvString(F("OK"), F("ERROR"), 5000);
    if (data.indexOf(F("OK")) != -1) {
        return true;
    }
    return false;
}

bool BiZduinoWiFi::eATCWLIF(String &list)
{
    String data;
    rx_empty();
    Serial.println(F("AT+CWLIF"));
    return recvFindAndFilter(F("OK"), F("\r\r\n"), F("\r\n\r\nOK"), list);
}
bool BiZduinoWiFi::eATCIPSTATUS(String &list)
{
    String data;
    delay(100);
    rx_empty();
    Serial.println(F("AT+CIPSTATUS"));
    return recvFindAndFilter(F("OK"), F("\r\r\n"), F("\r\n\r\nOK"), list);
}
bool BiZduinoWiFi::sATCIPSTARTSingle(String type, String addr, uint32_t port)
{
    String data;
    rx_empty();
    Serial.print(F("AT+CIPSTART=\""));
    Serial.print(type);
    Serial.print(F("\",\""));
    Serial.print(addr);
    Serial.print(F("\","));
    Serial.println(port);
    
    data = recvString(F("OK"), F("ERROR"), F("ALREADY CONNECT"), 10000);
    if (data.indexOf(F("OK")) != -1 || data.indexOf(F("ALREADY CONNECT")) != -1) {
        return true;
    }
    return false;
}
bool BiZduinoWiFi::sATCIPSTARTMultiple(uint8_t mux_id, String type, String addr, uint32_t port)
{
    String data;
    rx_empty();
    Serial.print(F("AT+CIPSTART="));
    Serial.print(mux_id);
    Serial.print(F(",\""));
    Serial.print(type);
    Serial.print(F("\",\""));
    Serial.print(addr);
    Serial.print(F("\","));
    Serial.println(port);
    
    data = recvString(F("OK"), F("ERROR"), F("ALREADY CONNECT"), 10000);
    if (data.indexOf(F("OK")) != -1 || data.indexOf(F("ALREADY CONNECT")) != -1) {
        return true;
    }
    return false;
}
bool BiZduinoWiFi::sATCIPSENDSingle(const uint8_t *buffer, uint32_t len)
{
    rx_empty();
    Serial.print(F("AT+CIPSEND="));
    Serial.println(len);
    if (recvFind(F(">"), 5000)) {
        rx_empty();
        for (uint32_t i = 0; i < len; i++) {
            Serial.write(buffer[i]);
        }
        return recvFind(F("SEND OK"), 10000);
    }
    return false;
}
bool BiZduinoWiFi::sATCIPSENDMultiple(uint8_t mux_id, const uint8_t *buffer, uint32_t len)
{
    rx_empty();
    Serial.print(F("AT+CIPSEND="));
    Serial.print(mux_id);
    Serial.print(F(","));
    Serial.println(len);
    if (recvFind(F(">"), 5000)) {
        rx_empty();
        for (uint32_t i = 0; i < len; i++) {
            Serial.write(buffer[i]);
        }
        return recvFind(F("SEND OK"), 10000);
    }
    return false;
}
bool BiZduinoWiFi::sATCIPCLOSEMulitple(uint8_t mux_id)
{
    String data;
    rx_empty();
    Serial.print(F("AT+CIPCLOSE="));
    Serial.println(mux_id);
    
    data = recvString(F("OK"), F("link is not"), 5000);
    if (data.indexOf(F("OK")) != -1 || data.indexOf(F("link is not")) != -1) {
        return true;
    }
    return false;
}
bool BiZduinoWiFi::eATCIPCLOSESingle(void)
{
    rx_empty();
    Serial.println(F("AT+CIPCLOSE"));
    return recvFind(F("OK"), 5000);
}
bool BiZduinoWiFi::eATCIFSR(String &list)
{
    rx_empty();
    Serial.println(F("AT+CIFSR"));
    return recvFindAndFilter(F("OK"), F("\r\r\n"), F("\r\n\r\nOK"), list);
}
bool BiZduinoWiFi::sATCIPMUX(uint8_t mode)
{
    String data;
    rx_empty();
    Serial.print(F("AT+CIPMUX="));
    Serial.println(mode);
    
    data = recvString(F("OK"), F("Link is builded"));
    if (data.indexOf(F("OK")) != -1) {
        return true;
    }
    return false;
}
bool BiZduinoWiFi::sATCIPSERVER(uint8_t mode, uint32_t port)
{
    String data;
    if (mode) {
        rx_empty();
        Serial.print(F("AT+CIPSERVER=1,"));
        Serial.println(port);
        
        data = recvString(F("OK"), F("no change"));
        if (data.indexOf(F("OK")) != -1 || data.indexOf(F("no change")) != -1) {
            return true;
        }
        return false;
    } else {
        rx_empty();
        Serial.println(F("AT+CIPSERVER=0"));
        return recvFind(F("\r\r\n"));
    }
}
bool BiZduinoWiFi::sATCIPSTO(uint32_t timeout)
{
    rx_empty();
    Serial.print(F("AT+CIPSTO="));
    Serial.println(timeout);
    return recvFind(F("OK"));
}




